import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Properties;

import lbi.utils.DBUtil;
import lbi.utils.IOUtil;

public class Dissimilarity {

	public static void main(String[] args) throws IOException, SQLException {
		// load properties
        Properties props = IOUtil.readProps("myJDBCdef-2016.props");
        // load drivers (all driver in properties thanks to loadJDBC DBUtil method)
        DBUtil.loadJDBC(props.getProperty("jdbcDriver"));
        // connect to data A table using A's credentials
        String url = props.getProperty("jdbcURL");
        String user = props.getProperty("userName");
        String pwd = props.getProperty("userPWD");    
        Connection con = DriverManager.getConnection(url, user, pwd);
        Statement stmt = con.createStatement();
        
        HashMap<Integer, String> cust_genders = new HashMap<Integer, String>(); //customer_gender
        HashMap<Integer, Integer> time_id2year = new HashMap<Integer, Integer>(); //the year of a certain time_id
        HashMap<Integer, HashSet<Integer>> store_distinct_customer_male = new HashMap<Integer, HashSet<Integer>>(); //distinct male customer per store
        HashMap<Integer, HashSet<Integer>> store_distinct_customer_female = new HashMap<Integer, HashSet<Integer>>(); //distinct female customer per store        
        
        //retrieve the customer gender associated at a certain customer_id
        String query_customer = "SELECT * FROM customer";
        ResultSet rs = stmt.executeQuery(query_customer);
        while(rs.next()){        	
        	Integer customer_id = rs.getInt("customer_id");
        	String customer_gender = rs.getString("gender");
        	cust_genders.put(customer_id, customer_gender);
        }
        
        //retrieve the year associated at a certain time_id
        String query_time2year = "SELECT * FROM time_by_day";
        rs = stmt.executeQuery(query_time2year);
        while(rs.next()){        	
        	Integer time_id = rs.getInt("time_id");
        	Integer time_year = rs.getInt("the_year");
        	time_id2year.put(time_id, time_year);
        }
        
        //scan all the sales
        String query_sales_fact = "SELECT * FROM sales_fact";
        rs = stmt.executeQuery(query_sales_fact);
        while(rs.next()){
        	Integer customer_id = rs.getInt("customer_id");
        	Integer store_id = rs.getInt("store_id");
        	Integer time_id = rs.getInt("time_id");        	        	
        	Integer year_sale = time_id2year.get(time_id);    
	
     	   	HashSet<Integer> hash_set_male = store_distinct_customer_male.get(store_id);
        	HashSet<Integer> hash_set_female = store_distinct_customer_female.get(store_id);
        	
        	/* FIRST TIME WE MEET THE STOREID */
        	if (hash_set_male == null && hash_set_female == null){ 
        			store_distinct_customer_male.put(store_id, new HashSet<Integer>());
        			store_distinct_customer_female.put(store_id, new HashSet<Integer>());
        			hash_set_male = store_distinct_customer_male.get(store_id);
        			hash_set_female = store_distinct_customer_female.get(store_id);
        	}
        	
        	if (year_sale == 1998){
        		String customer_gender = cust_genders.get(customer_id);
        		
        		if (customer_gender.equals("M")){        			        			
        			hash_set_male.add(customer_id);        			
        			
        		} else {        			        			
        			hash_set_female.add(customer_id);        			
        		}
        	}
        }
        
        Double dissimilarity = 0.0;
        int total_distinct_female = 0;
        int total_distinct_male = 0;
   
        for (Integer key : store_distinct_customer_male.keySet()){
        	int number_distinct_male = store_distinct_customer_male.get(key).size();
        	int number_distinct_female = store_distinct_customer_female.get(key).size();        	
        	
        	total_distinct_female += number_distinct_female;
        	total_distinct_male += number_distinct_male;
        	
        } 
        
        // calculate dissimilarity index
        for (Integer key : store_distinct_customer_male.keySet()){
        	int number_distinct_male = store_distinct_customer_male.get(key).size();
        	int number_distinct_female = store_distinct_customer_female.get(key).size();        	
        	double dis_index_store = (number_distinct_female*1.0/total_distinct_female) - (number_distinct_male*1.0/total_distinct_male);
        	dissimilarity += Math.abs(dis_index_store);        	
        }        
        dissimilarity /= 2;        
        System.out.println("The Dissimilarity Index of year 1998 is: "+dissimilarity);       
	}
}
